@@ -1,8 +1,9 @@
 #!/usr/bin/python
 #
 # COLOUR PICKER.
-# This is a lightly modified version of urwids palette_test.py example script as
-# found at https://raw.github.com/wardi/urwid/master/examples/palette_test.py
+# This is a lightly modified version of urwids palette_test.py example
+# script as found at
+# https://raw.github.com/wardi/urwid/master/examples/palette_test.py
 #
 # This version simply omits resetting the screens default colour palette,
 # and therefore displays the colour attributes as alot would render them in
@@ -31,10 +32,7 @@
 Palette test.  Shows the available foreground and background settings
 in monochrome, 16 color, 88 color and 256 color modes.
 """
-from __future__ import absolute_import
-
 import re
-import sys
 
 import urwid
 import urwid.raw_display
