@@ -16,8 +16,7 @@
 
 """Helpers for unittests themselves."""
 
-from __future__ import absolute_import
-
+import asyncio
 import functools
 import unittest
 
@@ -180,11 +179,12 @@ def make_ui(**kwargs):
     return ui
 
 
-def expected_failure(func):
-    """For marking expected failures for twisted.trial based unit tests.
+def async_test(coro):
+    """Run an asyncrounous test synchronously."""
 
-    The builtin unittest.expectedFailure does not work with twisted.trail,
-    there is an outstanding bug for this, but no one has ever fixed it.
-    """
-    func.todo = 'expected failure'
-    return func
+    @functools.wraps(coro)
+    def _actual(*args, **kwargs):
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(coro(*args, **kwargs))
+
+    return _actual
